"""*****************************************************************************
* Copyright (C) 2019 Microchip Technology Inc. and its subsidiaries.
*
* Subject to your compliance with these terms, you may use Microchip software
* and any derivatives exclusively with Microchip products. It is your
* responsibility to comply with third party license terms applicable to your
* use of third party software (including open source software) that may
* accompany Microchip software.
*
* THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER
* EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED
* WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A
* PARTICULAR PURPOSE.
*
* IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE,
* INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND
* WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS
* BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE
* FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN
* ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY,
* THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
*****************************************************************************"""
def onAttachmentConnected(source, target):
	ownerComponent = source["component"]
	print("USB HOST GEN Client Driver: USB Host Layer Connected")
	readValue = Database.getSymbolValue("usb_host", "CONFIG_USB_HOST_TPL_ENTRY_NUMBER")
	if readValue != None:
		args = {"nTpl": readValue + 1 }
		res = Database.sendMessage("usb_host", "UPDATE_TPL_ENTRY_NUMBER", args)
	
def onAttachmentDisconnected(source, target):
	ownerComponent = source["component"]
	print("USB HOST GEN Client Driver: USB Host Layer Disconnected")
	readValue = Database.getSymbolValue("usb_host", "CONFIG_USB_HOST_TPL_ENTRY_NUMBER")
	if readValue != None:
		args = {"nTpl": readValue - 1}
		res = Database.sendMessage("usb_host", "UPDATE_TPL_ENTRY_NUMBER", args)
		
def destroyComponent(component):	
	print("USB HOST GEN Client Driver: Destroyed")
	
def instantiateComponent(usbHostGenComponent):

	res = Database.activateComponents(["usb_host"])

	# USB Host GEN client driver instances 
	usbHostGenClientDriverInstance = usbHostGenComponent.createIntegerSymbol("CONFIG_USB_HOST_GEN_NUMBER_OF_INSTANCES", None)
	usbHostGenClientDriverInstance.setLabel("Number of GEN Host Driver Instances")
        helpText = '''Configure this to match the number of Communication Class
        Devices that the application should support.  For example, setting this
        to 3 will configure the application to simultaneously support 3
        different GEN or a composite USB device that has 3 GEN Interfaces.'''
        usbHostGenClientDriverInstance.setDescription(helpText)
        usbHostGenClientDriverInstance.setDefaultValue(1)
	usbHostGenClientDriverInstance.setVisible(True)
	
	#USB Host GEN Attach Listeners Number 
	usbHostGenClientDriverAttachListnerNumber = usbHostGenComponent.createIntegerSymbol("CONFIG_USB_HOST_GEN_ATTACH_LISTENERS_NUMBER", None)
	usbHostGenClientDriverAttachListnerNumber.setLabel("Number of GEN Host Attach Listeners")
        helpText = '''Configure this to define the maximum number of Attach
        Listener functions that can register with the GEN Client Driver. The
        driver will call all registered Attach Listener Functions for every GEN
        interface that has been detected.'''
	usbHostGenClientDriverAttachListnerNumber.setDescription(helpText)
	usbHostGenClientDriverAttachListnerNumber.setDefaultValue(1)
	usbHostGenClientDriverAttachListnerNumber.setVisible(True)


	##############################################################
	# system_definitions.h file for USB Host GEN Client driver   
	##############################################################
	usbHostGenSystemDefFile = usbHostGenComponent.createFileSymbol(None, None)
	usbHostGenSystemDefFile.setType("STRING")
	usbHostGenSystemDefFile.setOutputName("core.LIST_SYSTEM_DEFINITIONS_H_INCLUDES")
	usbHostGenSystemDefFile.setSourcePath("templates/host/system_definitions.h.host_gen_includes.ftl")
	usbHostGenSystemDefFile.setMarkup(True)
	
	##############################################################
	# system_config.h file for USB Host GEN Client driver   
	##############################################################
	usbHostGenSystemConfigFile = usbHostGenComponent.createFileSymbol(None, None)
	usbHostGenSystemConfigFile.setType("STRING")
	usbHostGenSystemConfigFile.setOutputName("core.LIST_SYSTEM_CONFIG_H_MIDDLEWARE_CONFIGURATION")
	usbHostGenSystemConfigFile.setSourcePath("templates/host/system_config.h.host_gen.ftl")
	usbHostGenSystemConfigFile.setMarkup(True)
	
	##############################################################
	# TPL Entry for GEN client driver 
	##############################################################
	usbHostGenTplEntryFile = usbHostGenComponent.createFileSymbol(None, None)
	usbHostGenTplEntryFile.setType("STRING")
	usbHostGenTplEntryFile.setOutputName("usb_host.LIST_USB_HOST_TPL_ENTRY")
	usbHostGenTplEntryFile.setSourcePath("templates/host/system_init_c_gen_tpl.ftl")
	usbHostGenTplEntryFile.setMarkup(True)
	
	################################################
	# USB Host GEN Client driver Files 
	################################################
	usbHostGenHeaderFile = usbHostGenComponent.createFileSymbol(None, None)
	addFileName('usb_host_gen.h', usbHostGenComponent, usbHostGenHeaderFile, "middleware/", "/usb/", True, None)
	
	usbGenHeaderFile = usbHostGenComponent.createFileSymbol(None, None)
	addFileName('usb_gen.h', usbHostGenComponent, usbGenHeaderFile, "middleware/", "/usb/", True, None)
	
	usbHostGenSourceFile = usbHostGenComponent.createFileSymbol(None, None)
	addFileName('usb_host_gen.c', usbHostGenComponent, usbHostGenSourceFile, "middleware/src/", "/usb/src/", True, None)
	
	usbHostGenAcmSourceFile = usbHostGenComponent.createFileSymbol(None, None)
	addFileName('usb_host_gen_acm.c', usbHostGenComponent, usbHostGenAcmSourceFile, "middleware/src/", "/usb/src/", True, None)
	
	usbHostGenLocalHeaderFile = usbHostGenComponent.createFileSymbol(None, None)
	addFileName('usb_host_gen_local.h', usbHostGenComponent, usbHostGenLocalHeaderFile, "middleware/src/", "/usb/src", True, None)
	
	
	# all files go into src/
def addFileName(fileName, component, symbol, srcPath, destPath, enabled, callback):
	configName1 = Variables.get("__CONFIGURATION_NAME")
	#filename = component.createFileSymbol(None, None)
	symbol.setProjectPath("config/" + configName1 + destPath)
	symbol.setSourcePath(srcPath + fileName)
	symbol.setOutputName(fileName)
	symbol.setDestPath(destPath)
	if fileName[-2:] == '.h':
		symbol.setType("HEADER")
	else:
		symbol.setType("SOURCE")
	symbol.setEnabled(enabled)
	if callback != None:
		symbol.setDependencies(callback, ["USB_DEVICE_FUNCTION_1_DEVICE_CLASS"])
